@@ -3,7 +3,7 @@
  */
 
 /* 
- * Copyright (C) 1986, 1988, 1989, 1991-2015 the Free Software Foundation, Inc.
+ * Copyright (C) 1986, 1988, 1989, 1991-2016 the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -237,6 +237,9 @@ do_fflush(int nargs)
 		fp = rp->output.fp;
 		if (fp != NULL)
 			status = rp->output.gawk_fflush(fp, rp->output.opaque);
+		else if ((rp->flag & RED_TWOWAY) != 0)
+				warning(_("fflush: cannot flush: two-way pipe `%s' has closed write end"),
+					file);
 	} else if ((fp = stdfile(tmp->stptr, tmp->stlen)) != NULL) {
 		status = fflush(fp);
 	} else {
@@ -657,7 +660,7 @@ format_tree(
 	int i, nc;
 	bool toofew = false;
 	char *obuf, *obufout;
-	size_t osiz, ofre;
+	size_t osiz, ofre, olen_final;
 	const char *chbuf;
 	const char *s0, *s1;
 	int cs1;
@@ -1569,7 +1572,10 @@ mpf1:
 			_("too many arguments supplied for format string"));
 	}
 	bchunk(s0, s1 - s0);
-	r = make_str_node(obuf, obufout - obuf, ALREADY_MALLOCED);
+	olen_final = obufout - obuf;
+	if (ofre > 0)
+		erealloc(obuf, char *, olen_final + 2, "format_tree");
+	r = make_str_node(obuf, olen_final, ALREADY_MALLOCED);
 	obuf = NULL;
 out:
 	{
@@ -1664,8 +1670,13 @@ do_printf(int nargs, int redirtype)
 		if (redir_exp->type != Node_val)
 			fatal(_("attempt to use array `%s' in a scalar context"), array_vname(redir_exp));
 		rp = redirect(redir_exp, redirtype, & errflg);
-		if (rp != NULL)
+		if (rp != NULL) {
+			if ((rp->flag & RED_TWOWAY) != 0 && rp->output.fp == NULL) {
+				(void) close_rp(rp, CLOSE_ALL);
+				fatal(_("printf: attempt to write to closed write end of two-way pipe"));
+			}
 			fp = rp->output.fp;
+		}
 	} else if (do_debug)	/* only the debugger can change the default output */
 		fp = output_fp;
 	else
@@ -2061,9 +2072,10 @@ NODE *
 do_system(int nargs)
 {
 	NODE *tmp;
-	int ret = 0;
+	AWKNUM ret = 0;		/* floating point on purpose, compat Unix awk */
 	char *cmd;
 	char save;
+	int status;
 
 	if (do_sandbox)
 		fatal(_("'system' function not allowed in sandbox mode"));
@@ -2080,11 +2092,47 @@ do_system(int nargs)
 		cmd[tmp->stlen] = '\0';
 
 		os_restore_mode(fileno(stdin));
-		ret = system(cmd);
-		if (ret != -1)
-			ret = WEXITSTATUS(ret);
+#ifdef SIGPIPE
+		signal(SIGPIPE, SIG_DFL);
+#endif
+
+		status = system(cmd);
+		/*
+		 * 3/2016. What to do with ret? It's never simple.
+		 * POSIX says to use the full return value. BWK awk
+		 * divides the result by 256.  That normally gives the
+		 * exit status but gives a weird result for death-by-signal.
+		 * So we compromise as follows:
+		 */
+		ret = status;
+		if (status != -1) {
+			if (do_posix)
+				;	/* leave it alone, full 16 bits */
+			else if (do_traditional)
+#ifdef __MINGW32__
+			  ret = (((unsigned)status) & ~0xC0000000);
+#else
+				ret = (status / 256.0);
+#endif
+			else if (WIFEXITED(status))
+				ret = WEXITSTATUS(status); /* normal exit */
+			else if (WIFSIGNALED(status)) {
+				bool coredumped = false;
+#ifdef WCOREDUMP
+				coredumped = WCOREDUMP(status);
+#endif
+				/* use 256 since exit values are 8 bits */
+				ret = WTERMSIG(status) +
+					(coredumped ? 512 : 256);
+			} else
+				ret = 0;	/* shouldn't get here */
+		}
+
 		if ((BINMODE & BINMODE_INPUT) != 0)
 			os_setbinmode(fileno(stdin), O_BINARY);
+#ifdef SIGPIPE
+		signal(SIGPIPE, SIG_IGN);
+#endif
 
 		cmd[tmp->stlen] = save;
 	}
@@ -2111,8 +2159,13 @@ do_print(int nargs, int redirtype)
 		if (redir_exp->type != Node_val)
 			fatal(_("attempt to use array `%s' in a scalar context"), array_vname(redir_exp));
 		rp = redirect(redir_exp, redirtype, & errflg);
-		if (rp != NULL)
+		if (rp != NULL) {
+			if ((rp->flag & RED_TWOWAY) != 0 && rp->output.fp == NULL) {
+				(void) close_rp(rp, CLOSE_ALL);
+				fatal(_("print: attempt to write to closed write end of two-way pipe"));
+			}
 			fp = rp->output.fp;
+		}
 	} else if (do_debug)	/* only the debugger can change the default output */
 		fp = output_fp;
 	else
@@ -2126,12 +2179,8 @@ do_print(int nargs, int redirtype)
 			fatal(_("attempt to use array `%s' in a scalar context"), array_vname(tmp));
 		}
 
-		if ((tmp->flags & (NUMBER|STRING)) == NUMBER) {
-			if (OFMTidx == CONVFMTidx)
-				args_array[i] = force_string(tmp);
-			else
-				args_array[i] = format_val(OFMT, OFMTidx, tmp);
-		}
+		if ((tmp->flags & STRCUR) == 0 || (tmp->stfmt != -1 && tmp->stfmt != OFMTidx))
+			args_array[i] = format_val(OFMT, OFMTidx, tmp);
 	}
 
 	if (redir_exp != NULL) {
@@ -2175,8 +2224,13 @@ do_print_rec(int nargs, int redirtype)
 	if (redirtype != 0) {
 		redir_exp = TOP();
 		rp = redirect(redir_exp, redirtype, & errflg);
-		if (rp != NULL)
+		if (rp != NULL) {
+			if ((rp->flag & RED_TWOWAY) != 0 && rp->output.fp == NULL) {
+				(void) close_rp(rp, CLOSE_ALL);
+				fatal(_("print: attempt to write to closed write end of two-way pipe"));
+			}
 			fp = rp->output.fp;
+		}
 		DEREF(redir_exp);
 		decr_sp();
 	} else
@@ -2818,8 +2872,10 @@ set_how_many:
 					leave alone, it goes into the output */
 			} else {
 				/* gawk default behavior since 1996 */
-				if (strncmp(scan, "\\\\\\&", 4) == 0) {
+				if (strncmp(scan, "\\\\\\&", 4) == 0
+				    || strncmp(scan, "\\\\\\\\", 4) == 0) {	/* 2016: fixed */
 					/* \\\& --> \& */
+					/* \\\\ --> \\ */
 					repllen -= 2;
 					scan += 3;
 				} else if (strncmp(scan, "\\\\&", 3) == 0) {
@@ -2924,10 +2980,12 @@ set_how_many:
 						*bp++ = *scan;
 					} else {
 						/* gawk default behavior since 1996 */
-						if (strncmp(scan, "\\\\\\&", 4) == 0) {
+						if (strncmp(scan, "\\\\\\&", 4) == 0
+						    || strncmp(scan, "\\\\\\\\", 4) == 0) {	/* 2016: fixed */
 							/* \\\& --> \& */
+							/* \\\\ --> \\ */
 							*bp++ = '\\';
-							*bp++ = '&';
+							*bp++ = scan[3];
 							scan += 3;
 						} else if (strncmp(scan, "\\\\&", 3) == 0) {
 							/* \\& --> \<string> */
@@ -3458,6 +3516,7 @@ nondec2awknum(char *str, size_t len)
 	} else {
 decimal:
 		save = str[len];
+		str[len] = '\0';
 		retval = strtod(str, NULL);
 		str[len] = save;
 	}
