@@ -3,7 +3,8 @@
  */
 
 /* 
- * Copyright (C) 1986, 1988, 1989, 1991-2013 the Free Software Foundation, Inc.
+ * Copyright (C) 1986, 1988, 1989, 1991-2013, 2016,
+ * the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -162,7 +163,7 @@ cint_array_init(NODE *symbol ATTRIBUTE_UNUSED, NODE *subs ATTRIBUTE_UNUSED)
 	} else
 		null_array(symbol);
 
-	return (NODE **) ! NULL;
+	return & success_node;
 }
 
 
@@ -172,7 +173,7 @@ NODE **
 is_uinteger(NODE *symbol, NODE *subs)
 {
 	if (is_integer(symbol, subs) != NULL && subs->numbr >= 0)
-		return (NODE **) ! NULL;
+		return & success_node;
 	return NULL;
 }
 
@@ -358,7 +359,7 @@ cint_remove(NODE *symbol, NODE *subs)
 		freenode(xn);
 	}
 
-	return (NODE **) ! NULL;
+	return & success_node;
 
 xremove:
 	xn = symbol->xarray;
@@ -371,7 +372,7 @@ xremove:
 	symbol->table_size--;
 	assert(symbol->table_size > 0);
 
-	return (NODE **) ! NULL;
+	return & success_node;
 }
 
 
