@@ -3,7 +3,7 @@
  */
 
 /* 
- * Copyright (C) 1986, 1988, 1989, 1991-2015 the Free Software Foundation, Inc.
+ * Copyright (C) 1986, 1988, 1989, 1991-2016 the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -122,7 +122,7 @@ char casetable[] = {
 	C('\360'), C('\361'), C('\362'), C('\363'), C('\364'), C('\365'), C('\366'), C('\367'),
 	C('\370'), C('\371'), C('\372'), C('\373'), C('\374'), C('\375'), C('\376'), C('\377'),
 };
-#elif 'a' == 0x81 /* it's EBCDIC */
+#elif defined(USE_EBCDIC)
 char casetable[] = {
  /*00  NU    SH    SX    EX    PF    HT    LC    DL */
       0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
@@ -215,11 +215,13 @@ load_casetable(void)
 	if (cp == NULL || strcmp(cp, "C") == 0 || strcmp(cp, "POSIX") == 0)
 		return;
 
-#ifndef ZOS_USS
+#ifndef USE_EBCDIC
 	/* use of isalpha is ok here (see is_alpha in awkgram.y) */
 	for (i = 0200; i <= 0377; i++) {
 		if (isalpha(i) && islower(i) && i != toupper(i))
 			casetable[i] = toupper(i);
+		else
+			casetable[i] = i;
 	}
 #endif
 #endif
@@ -949,6 +951,10 @@ set_LINT()
 	int old_lint = do_lint;
 	NODE *n = LINT_node->var_value;
 
+	/* start with clean defaults */
+	lintfunc = r_warning;
+	do_flags &= ~(DO_LINT_ALL|DO_LINT_INVALID);
+
 	if ((n->flags & (STRING|STRCUR)) != 0) {
 		if ((n->flags & MAYBE_NUM) == 0) {
 			const char *lintval;
@@ -958,38 +964,24 @@ set_LINT()
 			lintval = n->stptr;
 			lintlen = n->stlen;
 			if (lintlen > 0) {
-				do_flags |= DO_LINT_ALL;
-				if (lintlen == 5 && strncmp(lintval, "fatal", 5) == 0)
-					lintfunc = r_fatal;
-				else if (lintlen == 7 && strncmp(lintval, "invalid", 7) == 0) {
-					do_flags &= ~ DO_LINT_ALL;
+				if (lintlen == 7 && strncmp(lintval, "invalid", 7) == 0)
 					do_flags |= DO_LINT_INVALID;
-				} else
-					lintfunc = warning;
-			} else {
-				do_flags &= ~(DO_LINT_ALL|DO_LINT_INVALID);
-				lintfunc = warning;
+				else {
+					do_flags |= DO_LINT_ALL;
+					if (lintlen == 5 && strncmp(lintval, "fatal", 5) == 0)
+						lintfunc = r_fatal;
+				}
 			}
 		} else {
 			(void) force_number(n);
 			if (! iszero(n))
 				do_flags |= DO_LINT_ALL;
-			else
-				do_flags &= ~(DO_LINT_ALL|DO_LINT_INVALID);
-			lintfunc = warning;
 		}
 	} else if ((n->flags & (NUMCUR|NUMBER)) != 0) {
 		(void) force_number(n);
 		if (! iszero(n))
 			do_flags |= DO_LINT_ALL;
-		else
-			do_flags &= ~(DO_LINT_ALL|DO_LINT_INVALID);
-		lintfunc = warning;
-	} else
-		do_flags &= ~(DO_LINT_ALL|DO_LINT_INVALID);	/* shouldn't happen */
-
-	if (! do_lint)
-		lintfunc = warning;
+	}
 
 	/* explicitly use warning() here, in case lintfunc == r_fatal */
 	if (old_lint != do_lint && old_lint && ! do_lint)
@@ -1154,7 +1146,7 @@ r_get_lhs(NODE *n, bool reference)
 					array_vname(n));
 		if (n->orig_array->type != Node_var) {
 			n->orig_array->type = Node_var;
-			n->orig_array->var_value = Nnull_string;
+			n->orig_array->var_value = dupnode(Nnull_string);
 		}
 		/* fall through */
 	case Node_var_new:
