@@ -3,7 +3,7 @@
  */
 
 /* 
- * Copyright (C) 1986, 1988, 1989, 1991-2014 the Free Software Foundation, Inc.
+ * Copyright (C) 1986, 1988, 1989, 1991-2016 the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -1270,8 +1270,12 @@ choose_fs_function:
 			} else if (fs->stptr[0] == '\\') {
 				/* yet another special case */
 				strcpy(buf, "[\\\\\n]");
-			} else if (fs->stptr[0] != '\n')
+			} else if (fs->stptr[0] == '\0') {
+				/* and yet another special case */
+				strcpy(buf, "[\\000\n]");
+			} else if (fs->stptr[0] != '\n') {
 				sprintf(buf, "[%c\n]", fs->stptr[0]);
+			}
 		}
 	} else {
 		if (do_posix)
@@ -1598,9 +1602,8 @@ fpat_parse_field(long up_to,	/* parse only up to this field number */
 
 	if (in_middle) {
 		regex_flags |= RE_NO_BOL;
-		non_empty = rp->non_empty;
-	} else
-		non_empty = false;
+	}
+	non_empty = rp->non_empty;
 
 	eosflag = false;
 	need_to_set_sep = true;
