@@ -3,7 +3,7 @@
  */
 
 /* 
- * Copyright (C) 2012-2015 the Free Software Foundation, Inc.
+ * Copyright (C) 2012-2016 the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -82,7 +82,7 @@ api_get_argument(awk_ext_id_t id, size_t count,
 
 array:
 	/* get the array here */
-	arg = get_array_argument(count, false);
+	arg = get_array_argument(arg, count);
 	if (arg == NULL)
 		return awk_false;
 
@@ -90,7 +90,7 @@ array:
 
 scalar:
 	/* at this point we have a real type that is not an array */
-	arg = get_scalar_argument(count, false);
+	arg = get_scalar_argument(arg, count);
 	if (arg == NULL)
 		return awk_false;
 
@@ -120,7 +120,7 @@ api_set_argument(awk_ext_id_t id,
 	    || arg->type != Node_var_new)
 		return awk_false;
 
-	arg = get_array_argument(count, false);
+	arg = get_array_argument(arg, count);
 	if (arg == NULL)
 		return awk_false;
 
@@ -193,7 +193,7 @@ api_fatal(awk_ext_id_t id, const char *format, ...)
 	va_end(args);
 }
 
-/* api_warning --- print a warning message and exit */
+/* api_warning --- print a warning message */
 
 static void
 api_warning(awk_ext_id_t id, const char *format, ...)
@@ -219,11 +219,10 @@ api_lintwarn(awk_ext_id_t id, const char *format, ...)
 	va_start(args, format);
 	if (lintwarn == r_fatal) {
 		err(true, _("fatal: "), format, args);
-		va_end(args);
 	} else {
 		err(false, _("warning: "), format, args);
-		va_end(args);
 	}
+	va_end(args);
 }
 
 /* api_register_input_parser --- register an input_parser; for opening files read-only */
