@@ -3,7 +3,7 @@
  */
 
 /* 
- * Copyright (C) 1986, 1988, 1989, 1991-2015 the Free Software Foundation, Inc.
+ * Copyright (C) 1986, 1988, 1989, 1991-2016 the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -24,7 +24,7 @@
  */
 
 /* FIX THIS BEFORE EVERY RELEASE: */
-#define UPDATE_YEAR	2015
+#define UPDATE_YEAR	2016
 
 #include "awk.h"
 #include "getopt.h"
@@ -152,6 +152,7 @@ static int do_nostalgia = false;	/* provide a blast from the past */
 static int do_binary = false;		/* hands off my data! */
 static int do_version = false;		/* print version info */
 static const char *locale = "";		/* default value to setlocale */
+static char *locale_dir = LOCALEDIR;	/* default locale dir */
 
 int use_lc_numeric = false;	/* obey locale for decimal point */
 
@@ -219,6 +220,10 @@ main(int argc, char **argv)
 	char *extra_stack;
 	int have_srcfile = 0;
 	SRCFILE *s;
+	char *cp;
+#if defined(LOCALEDEBUG)
+	const char *initial_locale;
+#endif
 
 	/* do these checks early */
 	if (getenv("TIDYMEM") != NULL)
@@ -237,8 +242,13 @@ main(int argc, char **argv)
 	if (argc < 2)
 		usage(EXIT_FAILURE, stderr);
 
-	(void) bindtextdomain(PACKAGE, LOCALEDIR);
-	(void) textdomain(PACKAGE);
+	if ((cp = getenv("GAWK_LOCALE_DIR")) != NULL)
+		locale_dir = cp;
+
+#if defined(LOCALEDEBUG)
+	initial_locale = locale;
+#endif
+	set_locale_stuff();
 
 	(void) signal(SIGFPE, catchsig);
 #ifdef SIGBUS
@@ -286,7 +296,10 @@ main(int argc, char **argv)
 
 	parse_args(argc, argv);
 
-	set_locale_stuff();
+#if defined(LOCALEDEBUG)
+	if (locale != initial_locale)
+		set_locale_stuff();
+#endif
 
 	/*
 	 * In glibc, MB_CUR_MAX is actually a function.  This value is
@@ -1134,7 +1147,9 @@ arg_assign(char *arg, bool initing)
 		 */
 		if (do_posix)
 			setlocale(LC_NUMERIC, "C");
+#endif /* LC_NUMERIC */
 		(void) force_number(it);
+#ifdef LC_NUMERIC
 		if (do_posix)
 			setlocale(LC_NUMERIC, locale);
 #endif /* LC_NUMERIC */
@@ -1681,4 +1696,8 @@ set_locale_stuff(void)
 #if defined(LC_TIME)
 	setlocale(LC_TIME, locale);
 #endif
+
+	/* These must be done after calling setlocale */
+	(void) bindtextdomain(PACKAGE, locale_dir);
+	(void) textdomain(PACKAGE);
 }
