@@ -3,7 +3,7 @@
  */
 
 /* 
- * Copyright (C) 2012, 2013 the Free Software Foundation, Inc.
+ * Copyright (C) 2012, 2013, 2015 the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -39,6 +39,8 @@ mpz_t MFNR;
 bool do_ieee_fmt;	/* IEEE-754 floating-point emulation */
 mpfr_rnd_t ROUND_MODE;
 
+static mpfr_prec_t default_prec;
+
 static mpfr_rnd_t get_rnd_mode(const char rmode);
 static NODE *mpg_force_number(NODE *n);
 static NODE *mpg_make_number(double);
@@ -70,7 +72,7 @@ static inline mpfr_ptr mpg_tofloat(mpfr_ptr mf, mpz_ptr mz);
 void
 init_mpfr(mpfr_prec_t prec, const char *rmode)
 {
-	mpfr_set_default_prec(prec);
+	mpfr_set_default_prec(default_prec = prec);
 	ROUND_MODE = get_rnd_mode(rmode[0]);
 	mpfr_set_default_rounding_mode(ROUND_MODE);
 	make_number = mpg_make_number;
@@ -377,7 +379,7 @@ mpg_format_val(const char *format, int index, NODE *s)
 	} else {
 		r = format_tree(format, fmt_list[index]->stlen, dummy, 2);
 		assert(r != NULL);
-		s->stfmt = (char) index;
+		s->stfmt = index;
 	}
 	s->flags = oflags;
 	s->stlen = r->stlen;
@@ -525,7 +527,7 @@ set_PREC()
 	if ((val->flags & MAYBE_NUM) != 0)
 		force_number(val);
 
-	if ((val->flags & (STRING|NUMBER)) == STRING) {
+	if ((val->flags & STRCUR) != 0) {
 		int i, j;
 
 		/* emulate IEEE-754 binary format */
@@ -561,7 +563,7 @@ set_PREC()
 	}
 
 	if (prec > 0)
-		mpfr_set_default_prec(prec);
+		mpfr_set_default_prec(default_prec = prec);
 }
 
 
@@ -705,6 +707,7 @@ do_mpfr_func(const char *name,
 	NODE *t1, *res;
 	mpfr_ptr p1;
 	int tval;
+	mpfr_prec_t argprec;
 
 	t1 = POP_SCALAR();
 	if (do_lint && (t1->flags & (NUMCUR|NUMBER)) == 0)
@@ -713,7 +716,8 @@ do_mpfr_func(const char *name,
 	force_number(t1);
 	p1 = MP_FLOAT(t1);
 	res = mpg_float();
-	mpfr_set_prec(res->mpg_numbr, mpfr_get_prec(p1));	/* needed at least for sqrt() */
+	if ((argprec = mpfr_get_prec(p1)) > default_prec)
+		mpfr_set_prec(res->mpg_numbr, argprec);	/* needed at least for sqrt() */
 	tval = mpfr_func(res->mpg_numbr, p1, ROUND_MODE);
 	IEEE_FMT(res->mpg_numbr, tval);
 	DEREF(t1);
@@ -1214,9 +1218,17 @@ mpg_tofloat(mpfr_ptr mf, mpz_ptr mz)
 		prec -= (size_t) mpz_scan1(mz, 0);	/* least significant 1 bit index starting at 0 */
 		if (prec > MPFR_PREC_MAX)
 			prec = MPFR_PREC_MAX;
-		if (prec > PRECISION_MIN) 
-			mpfr_set_prec(mf, prec);
+		else if (prec < PRECISION_MIN)
+			prec = PRECISION_MIN;
 	}
+	else
+		prec = PRECISION_MIN;
+	/*
+	 * Always set the precision to avoid hysteresis, since do_mpfr_func 
+	 * may copy our precision.
+	 */
+	if (prec != mpfr_get_prec(mf))
+		mpfr_set_prec(mf, prec);
 
 	mpfr_set_z(mf, mz, ROUND_MODE);
 	return mf;
